"""
    Created on 2012-07-21
    @author: jldupont
"""

class ExcInfo(Exception): pass

class ExcWarn(Exception): pass

class ExcErr(Exception): pass

class ExcCrit(Exception): pass

class ExcQuit(Exception): pass
